//
// Library to keep track of time
//
//

// Include files

#include <avr/interrupt.h>
#include "time.h"

// Global variables

volatile unsigned long timer0_overflow_count=0;
volatile unsigned long timer0_millis=0;
static unsigned char timer0_fract=0;

// Functions

/* Initialize timer 0 for time tracking */

void init_time(void){
  TIMSK0=(1<<TOIE0);
  TCCR0A=(1<<WGM01)|(1<<WGM00);
  TCCR0B=(1<<CS01)|(1<<CS00);
  TCNT0=0;
  sei();
}

/* ISR for timer 0 */

ISR(TIMER0_OVF_vect){
  unsigned long m=timer0_millis;
  unsigned char f=timer0_fract;
  m += MILLIS_INC;
  f += FRACT_INC;
  if(f>=FRACT_MAX){ f -= FRACT_MAX; m += 1; }
  timer0_fract=f;
  timer0_millis=m;
  timer0_overflow_count++;
}

/* Return milliseconds */

unsigned long millis(void){
  unsigned long m;
  uint8_t oldSREG=SREG;
  cli();
  m=timer0_millis;
  SREG=oldSREG;
  return m;
}

/* Return microseconds */

unsigned long micros(void){
  unsigned long m;
  uint8_t oldSREG = SREG, t;
  cli();
  m=timer0_overflow_count;
  t=TCNT0;
  if((TIFR0&(1<<TOV0))&&(t<255)) m++;
  SREG = oldSREG;
  return ((m<<8)+t)*(64/clockCyclesPerMicrosecond());
}
